#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = 'cnheider'

import torch.nn as nn


def conv3x3(in_channels, out_channels, stride=1, padding=1, bias=True, groups=1):
  return nn.Conv2d(in_channels,
                   out_channels,
                   kernel_size=3,
                   stride=stride,
                   padding=padding,
                   bias=bias,
                   groups=groups)


def upconv2x2(in_channels, out_channels, mode='transpose'):
  if mode == 'transpose':
    return nn.ConvTranspose2d(in_channels,
                              out_channels,
                              kernel_size=2,
                              stride=2)
  else:
    # out_channels is always going to be the same as in_channels
    return nn.Sequential(nn.Upsample(mode='bilinear',
                                     scale_factor=2,
                                     align_corners=True),
                         conv1x1(in_channels,
                                 out_channels))


def conv1x1(in_channels, out_channels, groups=1):
  return nn.Conv2d(in_channels,
                   out_channels,
                   kernel_size=1,
                   groups=groups,
                   stride=1)
